/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef __HAL_BOARD_H__
#define __HAL_BOARD_H__

/*!
 * @addtogroup HAL_board_api
 * @{
 */

/*!
 * @brief MCLK frequency to run at.
 *
 * MCLK will be configured with the calibration values stored in INFO flash.
 * Acceptable values are 1000000, 8000000, 12000000 and 16000000.
 */
#define MCLK_FREQ               (1000000)

/*!
 * @brief Allow calls to the benchmark functions.
 */
#define ENABLE_BENCHMARK

/*!
 * @brief Set P1.1 high to start the benchmark.
 *
 * Set P1.1 high to start the benchmark.
 *
 * @return none
 */
#define board_benchmarkStart()  (P1OUT |= BIT1)

/*!
 * @brief Set P1.1 low to end the benchmark.
 *
 * Set P1.1 low to end the benchmark.
 *
 * @return none
 */
#define board_benchmarkStop()   (P1OUT &= ~BIT1)

/*!
 * @brief Return the state of the button switch.
 *
 * Return the state of the button switch.
 *
 * @return True if the button is pressed.
 */
#define board_buttonPressed()   (!(P1IN & BIT3))

/*!
 * @brief Delay by n milliseconds.
 *
 * Delay by n milliseconds.
 *
 * @param n Number of milliseconds to delay by.
 * @return none
 */
#define __delay_ms(n)           (__delay_cycles((n)*MCLK_FREQ/1000))

/*!
 * @brief Delay by n microseconds.
 *
 * Delay by n microseconds.
 *
 * @param n Number of microseconds to delay by.
 * @return none
 */
#define __delay_us(n)           (__delay_cycles((n)*MCLK_FREQ/1000000))

extern void board_init(void);
extern void board_gotoSleep(void);

/*! @} */

#endif //__HAL_BOARD_H__
