/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef __CALCULATION_H__
#define __CALCULATION_H__

/*!
 * @addtogroup calculation_api
 * @{
 */
 
/*!
 * @brief Structure definition for a 3D vector.
 *
 * 3D vector structure with x, y and z fields of type int16_t. The int16_t type
 * is equivalent to the _q data types and are interchangeable.
 */
typedef struct Vector3D {
    int16_t x;
    int16_t y;
    int16_t z;
}Vector3D;

/*!
 * @brief Threshold for drop detection.
 *
 * When in perfect free fall the magnitude of gravity will be zero. If any
 * rotation is present when the device is in free fall the magnitude will be
 * greater than zero due to the orientation of the sensor with respect to the
 * center of mass. This threshold can be adjusted to fine tune the sensitivity
 * of the drop detection. Increasing this value will increase the sensitivity
 * of the detection. 
 */
#define DROP_THRESHOLD          _Q12(0.2)

/*!
 * @brief Constant value for converting radians to degrees.
 */
#define RAD_TO_DEG              (57.29578)

extern void calculate_gravityVector(struct Vector3D *i16input, struct Vector3D *q12output);
extern void calculate_magnitude(struct Vector3D *q12vector, _q12 *q12magnitude);
extern void calculate_angles(struct Vector3D *q12vector, _q12 *q12pitch, _q12 *q12roll);
extern void calculate_dropDetection(_q12 *q12magnitude);

/*! @} */

#endif //__CALCULATION_H__
